/*
 * libtu/symlist.h
 *
 * Copyright (c) Tuomo Valkonen 1999-2004. 
 *
 * You may distribute and modify this library under the terms of either
 * the Clarified Artistic License or the GNU LGPL, version 2.1 or later.
 */

#ifndef LIBTU_SYMLIST_H
#define LIBTU_SYMLIST_H

#include "types.h"
#include "obj.h"


INTRSTRUCT(Symlist);


DECLSTRUCT(Symlist){
    void *symbol;
    Symlist *next, *prev;
};


typedef Symlist* SymlistIterTmp;

#define SYMLIST_FIRST(TYPE, LL) ((LL)==NULL ? NULL : (TYPE)(LL)->symbol)
#define SYMLIST_LAST(TYPE, LL) ((LL)==NULL ? NULL : (TYPE)(LL)->prev->symbol)

#define FOR_ALL_ON_SYMLIST(TYPE, VAR, LL, TMP) \
    for(symlist_iter_init(&(TMP), LL),         \
         VAR=(TYPE)symlist_iter(&(TMP));       \
        VAR!=NULL;                             \
        VAR=(TYPE)symlist_iter(&(TMP)))

#define FOR_ALL_ON_SYMLIST_REV(TYPE, VAR, LL, TMP) \
    for(symlist_iter_rev_init(&(TMP), LL),         \
         VAR=(TYPE)symlist_iter_rev(&(TMP));       \
        VAR!=NULL;                                 \
        VAR=(TYPE)symlist_iter_rev(&(TMP)))

#define FOR_ALL_ON_SYMLIST_UNSAFE(TYPE, VAR, LL) \
    FOR_ALL_ON_SYMLIST(TYPE, VAR, LL, symlist_iter_tmp)

extern SymlistIterTmp symlist_iter_tmp;

#define SYMLIST_EMPTY(LIST) ((LIST)==NULL)

extern bool symlist_insert_last(Symlist **symlist, void *symbol);
extern bool symlist_insert_first(Symlist **symlist, void *symbol);
extern bool symlist_reinsert_last(Symlist **symlist, void *symbol);
extern bool symlist_reinsert_first(Symlist **symlist, void *symbol);
extern void symlist_remove(Symlist **symlist, void *symbol);
extern void symlist_clear(Symlist **symlist);
extern void symlist_iter_init(SymlistIterTmp *state, Symlist *symlist);
extern void *symlist_iter(SymlistIterTmp *state);
extern void symlist_iter_rev_init(SymlistIterTmp *state, Symlist *symlist);
extern void *symlist_iter_rev(SymlistIterTmp *state);

#endif /* LIBTU_SYMLIST_H */
